module SkRequeue
  module Core

    def self.included(base)
      base.extend ClassMethods
      base.send :include, InstanceMethods
    end


    module ClassMethods

      def failure_and_duplicacy_recovery(action, enqueables_arr)
        ss = Sidekiq::ScheduledSet.new
        all_queued_keys = []
        enqueables_hash = enqueables_map(action, enqueables_arr)
        ss.each do |job|
          next if job.klass != worker_name
          sk_key = build_job_key(job)
          if all_queued_keys.include?(sk_key) || !(enqueables_hash.has_key?(sk_key))
            job.delete
          else
            all_queued_keys << sk_key
          end
        end
        detached_keys = (enqueables_hash.keys - all_queued_keys)
        detached_records = enqueables_hash.slice(*detached_keys).values
        yield(detached_records) if block_given?
        detached_records
      end

      def enqueables_map(action, enqueables_arr)
      	 temp_map = {}
     	 enqueables_arr.each do	|e| 
     	 	key = e.build_model_key(action) 
     	 	temp_map[key] = e
     	 end
     	 temp_map
  	  end

      def build_job_key(job)
        [job.args[0], job.args[1], job.args[2]].join(':')
      end

    end

    module InstanceMethods
      def requeue(action, *args)
        perform_uniq(action, worker_frequency, *args)
      end

      def requeue_in(action, proxy_frequency, *args)
        perform_uniq(action,proxy_frequency, *args)
      end

      def perform_uniq(action, sk_frequency , *args)
        ss = Sidekiq::ScheduledSet.new
        ss.each do |job|
          next if job.klass != klass.worker_name
          key = build_model_key(action)
          if klass.build_job_key(job) == key
            Rails.logger.warn("Skipping Current Request: A Job with key => #{key} already queued")
            return
          end
        end
        perform(next_enqueue_duration(sk_frequency), model_name, id, action, *args)
      end

      def build_model_key(action)
        key = "#{model_name}:#{id}:#{action}"
      end

      def next_enqueue_duration(frequency = nil)
        frequency ||= worker_frequency
        now = Time.zone.now + 5.hours + 30.minutes
        base_time = now.beginning_of_day
        while ((base_time - now) < 0)
          base_time = base_time +
            frequency
        end
        base_time - now
      end

      def perform(*args)
        worker_klass.perform_in(*args)
      end

      def worker_frequency_in_mins
        freq = self.try(klass.frequency_column.to_sym) || 60
      end

      def worker_frequency
        @worker_frequency ||= worker_frequency_in_mins * 60
      end

      def klass
        self.class
      end

      def worker_klass
        klass.worker_name.constantize
      end

      def model_name
        klass.to_s
      end

    end
  end
end
